import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'dart:async';

class TextUtils{

  static bool isEmpty(String text) => text == null || text.isEmpty || text.trim().isEmpty;
  static bool isNotEmpty(String text) => !isEmpty(text);
  static bool equals(String a, String b) => a == b;
}

abstract class AdeptController {

  bool get hasError;
  bool get isEmpty;
  bool get isNotEmpty => !isEmpty;
  String get error;
  set error(String value);
  void setError(String error, {bool notify});
  void clearError({bool notify});
  void clear();
}

class AdeptTextEditingController extends TextEditingController with AdeptController {
  String _error;

  AdeptTextEditingController({String text}) : super (text: text);

  @override
  bool get hasError => error != null;
  @override
  bool get isEmpty => text == null || text.isEmpty;
  @override
  String get error => _error;
  @override
  set error(String value) {
    _error = value;
    if(_error != null) _feedback();
  }
  @override
  void setError(String error, {bool notify = true}) {
    this.error = error;
    if(notify) notifyListeners();
  }
  @override
  void clearError({bool notify = true}) {
    _error = null;
    if(notify) notifyListeners();
  }

  void _feedback() async{
    for(int i = 0; i < 3; ++i)
      await Future.delayed(Duration(milliseconds: 100), () => HapticFeedback.vibrate());
  }

  @override
  void clear() {
    clearError();
    super.clear();
  }

}
class AdeptValueController<T> extends ValueNotifier<T> with AdeptController {
  String _error;

  AdeptValueController({T value}) : super (value);

  @override
  bool get hasError => error != null;
  @override
  bool get isEmpty => value == null;
  @override
  String get error => _error;
  @override
  set error(String value) {
    _error = value;
    if(_error != null) _feedback();
  }
  @override
  void setError(String error, {bool notify = true}) {
    this.error = error;
    if(notify) notifyListeners();
  }
  @override
  void clearError({bool notify = false}) {
    _error = null;
    if(notify) notifyListeners();
  }
  @override
  void clear() {
    clearError();
    value = null;
  }

  void _feedback() async{
    for(int i = 0; i < 3; ++i)
      await Future.delayed(Duration(milliseconds: 100), () => HapticFeedback.vibrate());
  }
}